;**********************************************************************
;    Filename:	    20time.asm                                       *
;    Date:          2004/9/25                                        *
;    File Version:                                                    *
;**********************************************************************

	list      		p=16F84             ; list directive to define processor
	#include 	<p16F84.inc>         ; processor specific variable definitions

	__CONFIG   _CP_OFF & _WDT_OFF & _PWRTE_ON & _XT_OSC

;***** VARIABLE DEFINITIONS
w_temp        	EQU     0x0C 	     ; variable used for context saving 
status_temp   	EQU     0x0D	     ; variable used for context saving

count_data		EQU	0x10		;1sec keisoku count ram
sec1_data		EQU	0x11		;1sec LED data ram
sec10_data		EQU	0x12		;10sec LED data ram
min1_data		EQU	0x13		;1min LED data ram
min10_data		EQU	0x14		;10min LED data ram
wait_data		EQU	0x15		;wait data ram
start_stop		EQU	0x16		;start/stop ram
									;start_stop,0  timer start/stop flg
									;start_stop,1  buzzer flg

key_data			EQU	0x17		;key data ram
									;key_data,2  start/stop key	
									;key_data,3  time20 key
									;key_data,4  time40 key
									;key_data,5  time60 key

key_input		EQU	0x18		;key input ram
									;key_input,2  start/stop key
									;key_input,3  time20 key
									;key_input,4  time40 key
									;key_input,5  time60 key
									
taihi			EQU	0x19		;taihi ram
led_count		EQU	0x1a		;led hyouji count
led_disp_0		EQU	0x1b		;led hyouji data 0
led_disp_1		EQU	0x1c		;led hyouji data 1
led_disp_2		EQU	0x1d		;led hyouji data 2
mode_count		EQU	0x1e		;mode count 0=none 1=20min 2=40min 3=60min
out_data			EQU	0x1f		;out data
dec_count		EQU	0x20		;dec count
led_data_taihi		EQU	0x21		;led data taihi
buzzer_time		EQU	0x22		;buzzer time
buzzer_time2	EQU	0x23		;buzzer time2

tm0_setdata		EQU	63h			;255-156=99=63H  156*128=19.968msec
cnt_setdata		EQU	32h			;19.968*50=0.9984sec  32h
;**********************************************************************
		ORG     		0x000            		 ; processor reset vector
  		goto   		 main             		 ; go to beginning of program

;----------------------------------------------------------------------------------
		ORG     		0x004            	 ; interrupt vector location
		movwf  	 w_temp           	 ; save off current W register contents
		movf		STATUS,w          ; move status register into W register
		movwf		status_temp    	 ; save off contents of STATUS register

		;---------------------------------	
		bcf			INTCON,T0IF
		movlw		tm0_setdata
		movwf		TMR0

		decfsz		count_data,1
		goto		next_int
time_up		
		movlw		cnt_setdata		
		movwf		count_data
		;--------------------------------		
		btfss		start_stop,0		;start/stop check
		goto		next_int

		call			set_led_data0
		movwf		led_disp_0
		call			set_led_data1
		movwf		led_disp_1
		call			set_led_data2
		movwf		led_disp_2

		call			zero_check
		btfsc		start_stop,1		;buzzer check
		goto		next_int

		btfss		sec1_data,0
		call			mode_check

		movlw		01h
		subwf		sec1_data,1
		btfsc		STATUS,0
		goto		next_int
		movlw		09h
		movwf		sec1_data
		
		movlw		01h
		subwf		sec10_data,1
		btfsc		STATUS,0
		goto		next_int
		movlw		05h
		movwf		sec10_data

		decf		dec_count,1
		btfss		STATUS,2		;Z check
		goto		next_int_s
		movf		mode_count,0
		movwf		dec_count
		decf		led_count,1
				
next_int_s
		movlw		01h
		subwf		min1_data,1
		btfsc		STATUS,0
		goto		next_int
		movlw		09h
		movwf		min1_data
		
		movlw		01h
		subwf		min10_data,1
		btfsc		STATUS,0
		goto		next_int
		movlw		09h
		movwf		min10_data

next_int
		nop
		;----------------------------------
		movf    		status_temp,w    	 ; retrieve copy of STATUS register
		movwf		STATUS           	 ; restore pre-isr STATUS register contents
		swapf   		w_temp,f
		swapf  		w_temp,w        	 ; restore pre-isr W register contents
		retfie                   			 ; return from interrupt
;---------------------------------------------------------------------------------------
main
		call			port_set
		movlw		07h				;port initial set
		movwf		PORTA
		movlw		0f0h
		movwf		PORTB

		call			timer0_set

		movlw		00h				;start/stop <= stop
									;buzzer flg off
		movwf		start_stop
		movlw		0ffh				;key data,key input clear
		movwf		key_data
		movwf		key_input

		movlw		cnt_setdata
		movwf		count_data
		call			sec_set
		call			min_clear
		
		movlw		01h
		movwf		mode_count
		call			set_20time		
		
		movlw		00h
		movwf		led_disp_0		;led_disp_0=00H
		movwf		led_disp_1		;led_disp_1=00H
		movlw		0f0h
		movwf		led_disp_2		;led_disp_2=F0H

main_loop
;-------------------------------------- key scan 1
		movlw		0ffh				;key input clear
		movwf		key_input
		btfss		PORTB,7		;start/stop key check
		bcf			key_input,2		;start/stop key input ari		
		btfss		PORTB,4		;20time key check
		bcf			key_input,3		;20rime key input ari
		btfss		PORTB,5		;40time key check
		bcf			key_input,4		;40time key input ari
		btfss		PORTB,6		;60time key check
		bcf			key_input,5		;60time key input ari
;---------------------------------------
		btfss		start_stop,0		;start/stop check
		call			mode_check
;------------------------------------- led_disp
		bsf			PORTB,2
		nop
		movf		led_disp_0,0
		call			data_out
		bcf			PORTB,2
		nop
		bcf			PORTA,0
		nop
		call			wait_1
		bsf			PORTA,0
		nop
		
		bsf			PORTB,2
		nop
		movf		led_disp_1,0
		call			data_out
		bcf			PORTB,2
		nop
		bcf			PORTA,1
		nop
		call			wait_1
		bsf			PORTA,1
		nop		
				
		bsf			PORTB,2
		nop
		movf		led_disp_2,0
		call			data_out
		bcf			PORTB,2
		nop
		bcf			PORTA,2
		nop
		call			wait_1
		bsf			PORTA,2
		nop
;------------------------------------- key scan 2
stsp_check
		btfss		PORTB,7		;start/stop key check
		goto		stsp_next
		bsf			key_data,2		;start/stop key off
		goto		sw20_check
stsp_next
		btfsc		key_input,2		;start/stop key input check
		goto		sw20_check
		btfss		key_data,2		;start/stop key on check
		goto		sw20_check
		bcf			key_data,2		;start/stop key on
		btfss		start_stop,0		;start/stop check
		goto		stsp_next_2
		bcf			start_stop,0		;start/stop <=stop
		nop
		bcf			start_stop,1		;buzzer flg off
		goto		sw20_check
stsp_next_2
		bsf			start_stop,0		;start/stop <=start		
;-------------------------------------
sw20_check
		btfss		PORTB,4		;sw20 key check
		goto		sw20_next
		bsf			key_data,3		;sw20 key off
		goto		sw40_check
sw20_next
		btfsc		key_input,3		;sw20 key input check
		goto		sw40_check
		btfss		key_data,3		;sw20 key on check
		goto		sw40_check
		bcf			key_data,3		;sw20 key on
		btfsc		start_stop,0		;start/stop check
		goto		sw40_check
		call			sec_set			;time20 set
		movlw		01h
		movwf		mode_count
		call			set_20time
		goto		next_step
;--------------------------------------
sw40_check
		btfss		PORTB,5		;sw40 key check
		goto		sw40_next
		bsf			key_data,4		;sw40 key off
		goto		sw60_check
sw40_next
		btfsc		key_input,4		;sw40 key input check
		goto		sw60_check
		btfss		key_data,4		;sw40 key on check
		goto		sw60_check
		bcf			key_data,4		;sw40 key on
		btfsc		start_stop,0		;start/stop check
		goto		sw60_check
		call			sec_set			;time40 set
		movlw		02h
		movwf		mode_count
		call			set_40time
		goto		next_step
;---------------------------------------
sw60_check
		btfss		PORTB,6		;sw60 key check
		goto		sw60_next
		bsf			key_data,5		;sw60 key off
		goto		next_step
sw60_next
		btfsc		key_input,5		;sw60 key input check
		goto		next_step
		btfss		key_data,5		;sw60 key on check
		goto		next_step
		bcf			key_data,5		;sw60 key on
		btfsc		start_stop,0		;start/stop check
		goto		next_step
		call			sec_set			;time60 set
		movlw		03h
		movwf		mode_count
		call			set_60time
		goto		next_step
;---------------------------------------
next_step


;---------------------------------------
buzzer
		btfss		start_stop,1		;buzzer flg check
		goto		buzzer_3
;-------
		call			buzzer_pi
		call			buzzer_wait
		btfss		start_stop,0		;start/stop check
		goto		buzzer_3
;-------
		movlw		0ffh				;key input clear
		movwf		key_input
		btfss		PORTB,7		;start/stop key check
		bcf			key_input,2		;start/stop key input ari			
;-------		
		btfss		PORTB,7		;start/stop key check
		goto		buzzer_4
		bsf			key_data,2		;start/stop key off
		goto		buzzer
buzzer_4
		btfsc		key_input,2		;start/stop key input check
		goto		buzzer
		btfss		key_data,2		;start/stop key on check
		goto		buzzer
		bcf			key_data,2		;start/stop key on
		bcf			start_stop,0		;start/stop <=stop
		nop
		bcf			start_stop,1		;buzzer flg off

buzzer_3
;---------------------------------------
		goto		main_loop
;-------------------------------------------------------------------------------
wait_3
		movlw		50h
		movwf		wait_data
		goto		wait_2

wait_1		
		movlw		0ffh
		movwf		wait_data
wait_2
		decfsz		wait_data,1
		goto		wait_2
		return
;----------------------------------------------------------------------------
buzzer_pi
		movlw		01h
		movwf		buzzer_time2
buzzer_pi1
		movlw		0c0h
		movwf		buzzer_time
buzzer_pi2
		bsf			PORTA,3
		call			wait_3
		bcf			PORTA,3
		call			wait_3
		decfsz		buzzer_time,1
		goto		buzzer_pi2
		decfsz		buzzer_time2,1
		goto		buzzer_pi1
		return
buzzer_wait
		movlw		01h
		movwf		buzzer_time2
buzzer_wait1
		movlw		0c0h
		movwf		buzzer_time
buzzer_wait2
		call			wait_3
		call			wait_3
		decfsz		buzzer_time,1
		goto		buzzer_wait2
		decfsz		buzzer_time2,1
		goto		buzzer_wait1
		return
;----------------------------------------------------------------------------
set_20time
		movlw		00h
		movwf		min1_data
		movlw		02h
		movwf		min10_data
set_time_e
		call			sec_set
		movlw		14h
		movwf		led_count
		movf		mode_count,0
		movwf		dec_count
		call			set_led_data0
		movwf		led_disp_0
		call			set_led_data1
		movwf		led_disp_1
		call			set_led_data2
		movwf		led_disp_2
		return
set_40time
		movlw		00h
		movwf		min1_data
		movlw		04h
		movwf		min10_data
		goto		set_time_e
set_60time
		movlw		00h
		movwf		min1_data
		movlw		06h
		movwf		min10_data
		goto		set_time_e
;-------------------------------------------------------------------------------
data_out
		movwf		out_data
		bcf			PORTB,1
		btfsc		out_data,7
		bsf			PORTB,1
		call			clk

		bcf			PORTB,1
		btfsc		out_data,6
		bsf			PORTB,1
		call			clk

		bcf			PORTB,1
		btfsc		out_data,5
		bsf			PORTB,1
		call			clk

		bcf			PORTB,1
		btfsc		out_data,4
		bsf			PORTB,1
		call			clk

		bcf			PORTB,1
		btfsc		out_data,3
		bsf			PORTB,1
		call			clk

		bcf			PORTB,1
		btfsc		out_data,2
		bsf			PORTB,1
		call			clk

		bcf			PORTB,1
		btfsc		out_data,1
		bsf			PORTB,1
		call			clk

		bcf			PORTB,1
		btfsc		out_data,0
		bsf			PORTB,1
		call			clk
		return
;--------------------------------------------------------------------------------
clk
		bsf			PORTB,0
		nop
		bcf			PORTB,0
		nop
		return
;--------------------------------------- mode_check
mode_check
		movf		mode_count,0
		btfsc		STATUS,2		;Z check
		goto		mode_check_e
		movf		mode_count,0
		sublw		01h		
		btfss		STATUS,2		;Z check
		goto		mode_check_2
		bcf			led_disp_2,4
		nop
		bsf			led_disp_2,5
		nop
		bsf			led_disp_2,6
		nop
		goto		mode_check_e
mode_check_2
		movf		mode_count,0
		sublw		02h		
		btfss		STATUS,2		;Z check
		goto		mode_check_3
		bsf			led_disp_2,4
		nop
		bcf			led_disp_2,5
		nop
		bsf			led_disp_2,6
		nop
		goto		mode_check_e
mode_check_3
		movf		mode_count,0
		sublw		03h		
		btfss		STATUS,2		;Z check
		goto		mode_check_e
		bsf			led_disp_2,4
		nop
		bsf			led_disp_2,5
		nop
		bcf			led_disp_2,6
		nop
		goto		mode_check_e		
mode_check_e
		return
;-----------------------------------------------------------------------------
set_led_data0
		movf		led_count,0
            movwf   	taihi
            movlw   	low(set_led_data0_table)
		addwf 		taihi,1
		movlw		high(set_led_data0_table)
		movwf		PCLATH
		addcf   		PCLATH,1
		movf    		taihi,0
		movwf   	PCL
set_led_data0_table   
		retlw		0ffh			;0
		retlw		0feh		;1
		retlw		0fch		;2
		retlw		0f8h		;3
		retlw		0f0h		;4
		retlw		0e0h		;5
		retlw		0c0h		;6
		retlw		80h			;7
		retlw		00h			;8
		retlw		00h			;9
		retlw		00h			;10
		retlw		00h			;11
		retlw		00h			;12
		retlw		00h			;13
		retlw		00h			;14
		retlw		00h			;15
		retlw		00h			;16
		retlw		00h			;17
		retlw		00h			;18
		retlw		00h			;19
		retlw		00h			;20
		
set_led_data1
		movf		led_count,0
            movwf   	taihi
            movlw   	low(set_led_data1_table)
		addwf 		taihi,1
		movlw		high(set_led_data1_table)
		movwf		PCLATH
		addcf   		PCLATH,1
		movf    		taihi,0
		movwf   	PCL
set_led_data1_table   
		retlw		0ffh			;0
		retlw		0ffh			;1
		retlw		0ffh			;2
		retlw		0ffh			;3
		retlw		0ffh			;4
		retlw		0ffh			;5
		retlw		0ffh			;6
		retlw		0ffh			;7
		retlw		0ffh			;8
		retlw		0feh		;9
		retlw		0fch		;10
		retlw		0f8h		;11
		retlw		0f0h		;12
		retlw		0e0h		;13
		retlw		0c0h		;14
		retlw		080h		;15
		retlw		00h			;16
		retlw		00h			;17
		retlw		00h			;18
		retlw		00h			;19
		retlw		00h			;20
																						
set_led_data2
		movf		led_count,0
            movwf   	taihi
            movlw   	low(set_led_data2_table)
		addwf 		taihi,1
		movlw		high(set_led_data2_table)
		movwf		PCLATH
		addcf   		PCLATH,1
		movf    		taihi,0
		movwf   	PCL
set_led_data2_table   
		retlw		0ffh			;0
		retlw		0ffh			;1
		retlw		0ffh			;2
		retlw		0ffh			;3
		retlw		0ffh			;4
		retlw		0ffh			;5
		retlw		0ffh			;6
		retlw		0ffh			;7
		retlw		0ffh			;8
		retlw		0ffh			;9
		retlw		0ffh			;10
		retlw		0ffh			;11
		retlw		0ffh			;12
		retlw		0ffh			;13
		retlw		0ffh			;14
		retlw		0ffh			;15
		retlw		0ffh			;16
		retlw		0feh		;17
		retlw		0fch		;18
		retlw		0f8h		;19
		retlw		0f0h		;20		

;-------------------------------------------------------------------------------
port_set				;RA0,RA1,RA2,RA3,RA4(OUT)
					;RB0,RB1,RB2,RB3(OUT)
					;RB4,RB5,RB6,RB7(IN)
		bsf			STATUS,RP0
		clrf			TRISA
		clrf			TRISB
		bsf			TRISB,4
		nop
		bsf			TRISB,5
		nop
		bsf			TRISB,6
		nop
		bsf			TRISB,7
		nop
		bcf			STATUS,RP0
		return
;-------------------------------------------------------------------------------
timer0_set
		bsf			STATUS,RP0
		movlw		086h		;(1)PORTB NOT PULLUP
								;(0)INTEDG
								;(0)TOCS
								;(0)TOSE
								;(0)PSA
								;(110)1/128
		movwf		OPTION_REG
		bcf			STATUS,RP0
		
		movlw		tm0_setdata
		movwf		TMR0
		bsf			INTCON,T0IE
		bsf			INTCON,GIE
		return
;------------------------------------------------------------------------------
sec_set
		movlw		09h
		movwf		sec1_data
		movlw		05h
		movwf		sec10_data
		return
;------------------------------------------------------------------------------
min_clear
		movlw		00h
		movwf		min1_data
		movwf		min10_data
		return
;------------------------------------------------------------------------------
zero_check
		movlw		09h
		subwf		sec1_data,0
		btfss		STATUS,0
		return
		movlw		05h
		subwf		sec10_data,0
		btfss		STATUS,0
		return
		movf		min1_data,1
		btfss		STATUS,2	;Z check
		return
		movf		min10_data,1
		btfss		STATUS,2	;Z check
		return
		btfss		start_stop,0	;start/stop check
		return
		bsf			start_stop,1	;buzzer flg on
		return
;-----------------------------------------------------------------------------
		END                     ; directive 'end of program'

